' ***************************************************************************************************
' SSMS Scripter
' A plugin for SQL Server 2005 Management Studio that allows you to script stored procedures,
' user-defined functions, and views in the SQL 2000 Enterprise Manager
' style: if exists, drop; create. 
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' ***************************************************************************************************

Imports System
Imports System.Collections
Imports System.Windows.Forms
Imports Microsoft.SqlServer.Management.UI.VSIntegration
Imports Microsoft.SqlServer.Management.UI.VSIntegration.ObjectExplorer
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Collections.Generic
Imports System.Reflection

<ComVisible(False)> _
Friend NotInheritable Class MenuNode

    ' *******************************************************************************************
    ' Enum:         MenuNodeType
    ' Author:       Mark Gilbert
    ' Date:         11/27/2006
    ' Description:  This enumerates the types of menu nodes in SSMS that the Scripter is designed
    '               to work with.
    ' Revisions:
    '   12/9/2006   Mark Gilbert added the "Other" item
    '   12/11/2006  Mark Gilbert added the "Database" item
    '   12/23/2006  Mark Gilbert changed the name from ObjectTypeEnum to MenuNodeTypeEnum, and
    '                   moved it from Common.vb to here.
    '   12/25/2006  Mark Gilbert changed the name from MenuNodeTypeEnum to simply MenuNodeType.
    '                   He also changed the declaration from Public to Friend.
    ' *******************************************************************************************
    Friend Enum MenuNodeType
        Other = -1
        Views = 0
        StoredProcedures = 1
        UserDefinedFunctions = 2
        Database = 3
    End Enum

    Friend Const ALL_OBJECTS As String = "*"


    Private m_plgMain As PlugIn
    Private m_strObjectName As String
    Private m_strDatabase As String
    Private m_blnIsSproc As Boolean
    Private m_blnIsSProcsFolder As Boolean
    Private m_blnIsView As Boolean
    Private m_blnIsViewFolder As Boolean
    Private m_blnIsUDF As Boolean
    Private m_blnIsUDFFolder As Boolean
    Private m_blnIsDatabaseFolder As Boolean
    Private m_blnIsOtherNode As Boolean
    Private m_mntCurrent As MenuNodeType
    Private m_connCurrent As Microsoft.SqlServer.Management.Common.SqlOlapConnectionInfoBase


#Region " Properties "
    ' *******************************************************************************************
    ' Property:     MenuNode class properties
    ' Author:       Mark Gilbert
    ' Date:         11/26/2006
    ' Parameters:   {none}
    ' Description:  These expose the private variables declared above.
    ' Revisions:
    '   11/27/2006  Mark Gilbert added the IsAllSProcNode, IsViewNode, IsViewFolder, IsUDFNode,
    '                   and IsUDFFolder properties.
    '
    '                   He also modified the ObjectType parameter to actually return the type of
    '                   the node and not just a hardcoded Common.ObjectTypeEnum.StoredProcedure
    '                   value.
    '   12/22/2006  Mark Gilbert added the Connection property, and removed the ServerName property.
    '   12/23/2006  Mark Gilbert changed the ObjectType property to MenuNodeType.
    ' *******************************************************************************************
    Friend ReadOnly Property DatabaseName() As String
        Get
            Return Me.m_strDatabase
        End Get
    End Property

    Friend ReadOnly Property NodeType() As MenuNodeType
        Get
            Return Me.m_mntCurrent
        End Get
    End Property

    Friend ReadOnly Property ObjectName() As String
        Get
            Return Me.m_strObjectName
        End Get
    End Property

    Private ReadOnly Property IsSProcNode() As Boolean
        Get
            Return Me.m_blnIsSproc
        End Get
    End Property

    Private ReadOnly Property IsSProcFolder() As Boolean
        Get
            Return Me.m_blnIsSProcsFolder
        End Get
    End Property

    Private ReadOnly Property IsViewNode() As Boolean
        Get
            Return Me.m_blnIsView
        End Get
    End Property

    Private ReadOnly Property IsViewFolder() As Boolean
        Get
            Return Me.m_blnIsViewFolder
        End Get
    End Property

    Private ReadOnly Property IsUDFNode() As Boolean
        Get
            Return Me.m_blnIsUDF
        End Get
    End Property

    Private ReadOnly Property IsUDFFolder() As Boolean
        Get
            Return Me.m_blnIsUDFFolder
        End Get
    End Property

    Private ReadOnly Property IsOtherNode() As Boolean
        Get
            Return Me.m_blnIsOtherNode
        End Get
    End Property

    Friend ReadOnly Property Connection() As Microsoft.SqlServer.Management.Common.SqlOlapConnectionInfoBase
        Get
            Return Me.m_connCurrent
        End Get
    End Property


    ' *******************************************************************************************
    ' Property:     IsValidNode
    ' Author:       Mark Gilbert
    ' Date:         11/27/2006
    ' Parameters:   {none}
    ' Description:  This returns True if the node represents a valid object to be scripted, and
    '               False otherwise.
    ' *******************************************************************************************
    Friend ReadOnly Property IsValidNode() As Boolean
        Get
            Return Me.m_blnIsSproc Or _
                    Me.m_blnIsSProcsFolder Or _
                    Me.m_blnIsView Or _
                    Me.m_blnIsViewFolder Or _
                    Me.m_blnIsUDF Or _
                    Me.m_blnIsUDFFolder Or _
                    Me.m_blnIsDatabaseFolder
        End Get
    End Property
#End Region


    ' *******************************************************************************************
    ' Method:       New
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   node - the SSMS node selected
    '               plugin - the plugin object that is interfacing with SSMS
    ' Description:  This constructor creates a new MenuNode object
    ' Revisions:
    '   11/27/2006  Mark Gilbert modified the code to check for adding the new menu at both the
    '                   Stored Procedure folder level, and the individual stored procedure level.
    '                   He also refactored the code quite a bit.
    '   12/9/2006   Mark Gilbert factored out the "*" used to signify "all objects", and replaced
    '                   it with the Common.ALL_OBJECTS constant
    '   12/11/2006  Mark Gilbert modified the constructor to add a menu item to the Database level.
    '   12/22/2006  Mark Gilbert modified the code to use the niCurrent.Connection property
    '                   to save the authentication credentials of the currently selected MenuNode
    '                   being created.  Since ServerName is part of these credentials already,
    '                   he also modified the code to not save this off individually.
    '   12/23/2006  Mark Gilbert changed the references to Common.ObjectTypeEnum to MenuNodeTypeEnum.
    '   2/13/2007   Mark Gilbert modified the constructor to specifically look for nodes selected
    '                   by the user that are invalid (meaning nodes that the Scripter doesn't
    '                   support).  If an invalid node has been selected, the code exits early.
    '                   Previously, selecting an invalid node (like a Table) would cause the
    '                   Scripter to register itself with that node's context menu because of the
    '                   "catch-all" clause that was originally designed to register the Scripter
    '                   with the Database node.
    ' *******************************************************************************************
    Private Sub New(ByVal niCurrent As INodeInformation, ByVal plgCurrent As PlugIn)
        Dim blnDoServerDatabaseElementsExist As Boolean
        Dim blnDoObjectNameSchemaElementsExist As Boolean
        Dim strCurrentContext As String
        Dim blnIsUDFScaler As Boolean
        Dim blnIsUDFTable As Boolean

        Me.m_plgMain = plgCurrent

        ' ***************************************************************************************
        ' If the current node is Nothing, then clear out the flags and exit
        If (IsNothing(niCurrent)) Then
            Me.m_blnIsSproc = False
            Me.m_blnIsSProcsFolder = False
            Me.m_blnIsView = False
            Me.m_blnIsViewFolder = False
            Me.m_blnIsUDF = False
            Me.m_blnIsViewFolder = False
            Me.m_blnIsDatabaseFolder = False
            Exit Sub
        End If


        ' ***************************************************************************************
        ' See what kind of node the user selected.  Possible entities that can be selected (the strings shown
        ' here are stored in niCurrent.Context):
        '   Database:
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Northwind']
        '   Single Stored Procedure:
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Northwind']/StoredProcedure[@Name='CustOrderHist' and @Schema='dbo']
        '   Single View:
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Northwind']/View[@Name='Invoices' and @Schema='dbo']
        '   Single User-Defined Function:
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Eaton_IPC']/UserDefinedFunction[@Name='fn_Split' and @Schema='dbo']
        '   Folder - "Stored Procedures":
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Northwind']/Folder[@Name='StoredProcedures' and @Type='StoredProcedure']
        '   Folder - "Views":
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Northwind']/Folder[@Name='Views' and @Type='View']
        '   Folder - "Functions":
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Eaton_IPC']/Folder[@Name='UsrDbFunctions']
        '   Folder - "Table-valued Functions":
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Eaton_IPC']/Folder[@Name='Table-valuedFunctions' and @Type='UserDefinedFunction']
        '   Folder - "Scalar-valued Functions":
        '           Server[@Name='MGILBERT-L-XP']/Database[@Name='Eaton_IPC']/Folder[@Name='Scalar-valuedFunctions' and @Type='UserDefinedFunction']
        strCurrentContext = niCurrent.Context


        ' ***************************************************************************************
        ' All elements need to have the Server Name and Database Name elements present
        blnDoServerDatabaseElementsExist = strCurrentContext.Contains("Server[@Name='") AndAlso strCurrentContext.Contains("']/Database[@Name='")

        ' Individual stored procedures, views, and UDF's need to have the Object Name and Schema elements present
        blnDoObjectNameSchemaElementsExist = strCurrentContext.Contains("@Schema") AndAlso _
                                            (Not IsNothing(niCurrent("Name"))) AndAlso _
                                            (Not IsNothing(niCurrent("Schema"))) AndAlso _
                                            (niCurrent("Name").ToString().Trim() <> "") AndAlso _
                                            (niCurrent("Schema").ToString().Trim() <> "")

        ' Set some specific internal variables to denote the node's type.
        Me.m_blnIsSproc = blnDoServerDatabaseElementsExist AndAlso _
                        blnDoObjectNameSchemaElementsExist AndAlso _
                        strCurrentContext.Contains("/StoredProcedure[@Name")

        Me.m_blnIsSProcsFolder = blnDoServerDatabaseElementsExist AndAlso _
                        strCurrentContext.Contains("/Folder[@Name='StoredProcedures' and @Type='StoredProcedure']")

        Me.m_blnIsView = blnDoServerDatabaseElementsExist AndAlso _
                        blnDoObjectNameSchemaElementsExist AndAlso _
                        strCurrentContext.Contains("/View[@Name")

        Me.m_blnIsViewFolder = blnDoServerDatabaseElementsExist AndAlso _
                        strCurrentContext.Contains("/Folder[@Name='Views' and @Type='View']")

        ' TODO: Issue with attaching a menu to the scaler or table-valued UDFs.
        ' Since the UDFs are broken out by Scaler and Table-Valued, I need two flags in PlugIn to track when menus
        ' have been added.  That part is easy enough.  The problem is knowing what type of UDF niCurrent is when the
        ' menu item is being registered.  A UDFs parent is the database, not the next folder up in the hierarchy.
        '
        ' As a result of this issue, no menu items will be added for individual UDFs.  Only the UDF folder will get
        ' the menu item.  That is why I have hardcoded Me.m_blnIsUDF to be False here.
        '                                                                                   -Mark Gilbert, 11/27/2006
        'Me.m_blnIsUDF = blnDoServerDatabaseElementsExist AndAlso _
        '                blnDoObjectNameSchemaElementsExist AndAlso _
        '                strCurrentContext.Contains("/UserDefinedFunction[@Name")
        Me.m_blnIsUDF = False


        Me.m_blnIsUDFFolder = blnDoServerDatabaseElementsExist AndAlso _
                        strCurrentContext.Contains("/Folder[@Name='UsrDbFunctions']")


        ' Check to see if this is any other type of node; if so, exit now
        Me.m_blnIsOtherNode = False
        Me.m_blnIsOtherNode = Not Me.IsValidNode And _
                        (strCurrentContext.Contains("/Folder[@Name") Or _
                        strCurrentContext.Contains("/Table[@Name"))
        If (Me.IsOtherNode) Then Return


        ' If this isn't a valid node yet (but it also doesn't fall into the "other node" category), then check to
        ' see if the user has selected a database folder.  If the current node isn't a database folder either, then
        ' it isn't one we need to be concerned about, so skip the rest of the constructor.
        '
        ' Because the database folder's Context string is a subset of all of the other ones, checking for this
        ' first would mean that selecting any other valid node would also set this property to be True.  Checking
        ' it last, after the current node has been evaluated against every other valid node type, means that only
        ' when the user REALLY has selected a database node will this property be set to True.
        If (Not Me.IsValidNode) Then
            Me.m_blnIsDatabaseFolder = blnDoServerDatabaseElementsExist

            If (Not Me.m_blnIsDatabaseFolder) Then Exit Sub
        End If


        ' ***************************************************************************************
        ' Now, pull off the common properties of the currently selected object, and register the
        ' menu item if it hasn't been already
        Me.m_connCurrent = niCurrent.Connection

        ' Split the Context property by single quotes.  The Database name will be the fourth item in the array.
        Me.m_strDatabase = niCurrent.Context.Split("'")(3)

        ' Now, store the object name and type (if applicable), and add the menu item if it hasn't been already
        If (Me.m_blnIsDatabaseFolder) Then
            ' ***************************************************************************************************
            ' DEVELOPER'S NOTE
            ' This item needs to be the first check in the IF-THEN structure because the niCurrent.Context
            ' string for a database folder is a subset of all of the other ones there.
            ' ***************************************************************************************************
            Me.m_strObjectName = ""
            Me.m_mntCurrent = MenuNodeType.Database

            ' If the menu has already been registered for the Database folder, then don't do it again.
            If (Not plgCurrent.IsMenuRegisteredForDatabase) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForDatabase = True
            End If

        ElseIf (Me.m_blnIsSProcsFolder) Then
            Me.m_strObjectName = MenuNode.ALL_OBJECTS
            Me.m_mntCurrent = MenuNodeType.StoredProcedures

            ' If the menu has already been registered for the Stored Procedure folder, then don't do it again.
            If (Not plgCurrent.IsMenuRegisteredForSProcFolder) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForSProcFolder = True
            End If

        ElseIf (Me.m_blnIsSproc) Then
            Me.m_strObjectName = niCurrent("Name").ToString()
            Me.m_mntCurrent = MenuNodeType.StoredProcedures

            ' If the menu has already been registered for a specific stored procedure, then it will be registered for all of them
            If (Not plgCurrent.IsMenuRegisteredForSProcNode) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForSProcNode = True
            End If

        ElseIf (Me.m_blnIsView) Then
            Me.m_strObjectName = niCurrent("Name").ToString()
            Me.m_mntCurrent = MenuNodeType.Views

            ' If the menu has already been registered for a specific view, then it will be registered for all of them
            If (Not plgCurrent.IsMenuRegisteredForViewNode) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForViewNode = True
            End If

        ElseIf (Me.m_blnIsViewFolder) Then
            Me.m_strObjectName = MenuNode.ALL_OBJECTS
            Me.m_mntCurrent = MenuNodeType.Views

            ' If the menu has already been registered for the Views folder, then don't do it again.
            If (Not plgCurrent.IsMenuRegisteredForViewFolder) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForViewFolder = True
            End If

        ElseIf (Me.m_blnIsUDF) Then
            ' ***************************************************************************************************
            ' DEVELOPER'S NOTE
            ' This clause will never be reached.  Please see the TODO comment that appears above the
            ' Me.m_blnIsUDF assignment for the reason why.
            ' ***************************************************************************************************
            Me.m_strObjectName = niCurrent("Name").ToString()
            Me.m_mntCurrent = MenuNodeType.UserDefinedFunctions

            ' If the menu has already been registered for a specific scaler UDF, then it will be registered for all of them
            If (blnIsUDFScaler And Not plgCurrent.IsMenuRegisteredForUDFScalerNode) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForUDFScalerNode = True
            End If

            ' If the menu has already been registered for a specific table UDF, then it will be registered for all of them
            If (blnIsUDFTable And Not plgCurrent.IsMenuRegisteredForUDFTableNode) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForUDFTableNode = True
            End If

        ElseIf (Me.m_blnIsUDFFolder) Then
            Me.m_strObjectName = MenuNode.ALL_OBJECTS
            Me.m_mntCurrent = MenuNodeType.UserDefinedFunctions

            ' If the menu has already been registered for the UDF folder, then don't do it again.
            If (Not plgCurrent.IsMenuRegisteredForUDFFolder) Then
                Me.AddScripterMenuItem(niCurrent)
                plgCurrent.IsMenuRegisteredForUDFFolder = True
            End If

        End If
    End Sub


    ' *******************************************************************************************
    ' Method:       GetInstanceOnSelectedNode
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   plugin
    ' Description:  This function returns a TableMenuNode object built on the properties of the
    '               currently selected table node in SSMS.
    ' Revisions:
    '   11/27/2006  Mark Gilbert modified the Return statement to also return the node when the
    '                   user has selected the "Stored Procedures" folder.  He also changed
    '                   SProcNodesDict to simply NodesDict, to be more generic.
    ' *******************************************************************************************
    Friend Shared Function GetInstanceOnSelectedNode(ByVal plgCurrent As PlugIn) As MenuNode
        Dim niCurrent As INodeInformation
        Dim intArraySize As Integer
        Dim niSelectedArray As INodeInformation()

        niSelectedArray = Nothing
        plgCurrent.ObjectExplorer.GetSelectedNodes(intArraySize, niSelectedArray)

        If (intArraySize > 0) Then
            niCurrent = niSelectedArray(0)

            If (Not IsNothing(niCurrent)) Then
                Dim strContext As String = niCurrent.Context
                Dim mnCurrent As MenuNode

                If (plgCurrent.NodesDict.ContainsKey(strContext)) Then
                    mnCurrent = plgCurrent.NodesDict(strContext)
                Else
                    mnCurrent = New MenuNode(niCurrent, plgCurrent)
                    plgCurrent.NodesDict.Add(strContext, mnCurrent)
                End If

                Return IIf(mnCurrent.IsValidNode, mnCurrent, Nothing)
            End If

        End If
        Return Nothing
    End Function



    ' *******************************************************************************************
    ' Method:       AddScripterMenuItem
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   niCurrent - the current node selected in SSMS
    ' Description:  This routine and AddMenuItem adds a menu item for the custom scripter to SSMS
    ' Revisions:
    '   1/27/2007   Mark Gilbert made a slight correction to account for SQL Server 2005 Service
    '                   Pack 2 CTP.  Please see the comments below for details of the issue and
    '                   the resolution.
    ' *******************************************************************************************
    Private Sub AddScripterMenuItem(ByVal niCurrent As INodeInformation)
        Dim strCompany, strProduct, strMenuText As String
        Dim objParameters As Object() = New Object() {}
        Dim attrCustom As Attribute

        ' Iterate through all the attributes for the assembly, and find the Company and Product attributes
        strCompany = ""
        strProduct = ""
        For Each attrCustom In Attribute.GetCustomAttributes(System.Reflection.Assembly.GetExecutingAssembly)
            If TypeOf attrCustom Is AssemblyCompanyAttribute Then
                strCompany = CType(attrCustom, AssemblyCompanyAttribute).Company
            End If

            If TypeOf attrCustom Is AssemblyProductAttribute Then
                strProduct &= CType(attrCustom, AssemblyProductAttribute).Product
            End If
        Next
        strMenuText = strCompany & " " & strProduct
        strMenuText = strMenuText.Trim()


        ' This declaration may cause problems.  Here is the original C#:
        '       Type[] typeArray = new Type[0];
        ' And this is the VB version that failed with this error "'New' cannot be used on a class that is declared as 'MustInherit'":
        '       Dim typeArray As Type() = New Type(0)
        Dim typeArray As Array = Array.CreateInstance(GetType(Type), 0)

        ' I also had problems with this declaration/assignment.
        '
        ' Original C#:
        '       MenuItem[] menuItems = (MenuItem[])menu.GetType().GetMethod("GetMenuItems", typeArray).Invoke(menu, prms);
        ' Original VB Port:
        '       Dim menuItems As MenuItem() = CType(Menu.GetType().GetMethod("GetMenuItems", typeArray).Invoke(Menu, prms), MenuItem())
        '
        ' What I found was that the VB compiler did not recognize a GetType() method for IMenuHandler, but it worked fine
        ' in the C# version.  Through trial and error I found that there was a GetType() method recognized for
        ' niCurrent.GetService(GetType(IMenuHandler)), so I modified AddScripterMenuItem to take an INodeInformation object,
        ' do the conversion to IMenuHandler here, and then do the rest of the line.
        '                                                                               - Mark Gilbert, 11/26/2006
        '
        ' With the release of SQL Server 2005 Service Pack 2 CTP, I found that this line no longer worked:
        '       Dim miCurrent As MenuItem() = CType(niCurrent.GetService(GetType(IMenuHandler)).GetType().GetMethod("GetMenuItems", typeArray).Invoke(niCurrent.GetService(GetType(IMenuHandler)), objParameters), MenuItem())
        ' The error I was getting was to the effect of "cannot cast object of type ToolStripItem to MenuItem".  The cast was the very last thing
        ' that this line was doing.  Since I wasn't actually using the miCurrent object, I extracted what was going on within the CType()
        ' function, and used it here, as a standalone command.  That got past the error, and it seems to accomplish the same thing.
        '                                                                               - Mark Gilbert, 1/27/2007
        niCurrent.GetService(GetType(IMenuHandler)).GetType().GetMethod("GetMenuItems", typeArray).Invoke(niCurrent.GetService(GetType(IMenuHandler)), objParameters)

        Me.AddMenuItem(niCurrent, "-")
        Me.AddMenuItem(niCurrent, strMenuText)
    End Sub


    ' *******************************************************************************************
    ' Method:       AddMenuItem
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   niCurrent - the current node selected in SSMS
    '               itenName - the item to insert into the menu structure
    ' Description:  This routine and AddScripterMenuItem adds a menu item for the custom scripter
    '               to SSMS.
    ' *******************************************************************************************
    Private Sub AddMenuItem(ByVal niCurrent As INodeInformation, ByVal strMenuItemToAdd As String)
        Dim miNew As ScripterMenuItem = New ScripterMenuItem(Me.m_plgMain)
        miNew.Text = strMenuItemToAdd
        Dim objParameters As Object() = New Object() {"", miNew}

        ' I had problems with this declaration/assignment.
        '
        ' Original C#:
        '       menu.GetType().GetMethod("AddChild").Invoke(menu, prms);
        ' Original VB Port:
        '       menu.GetType().GetMethod("AddChild").Invoke(menu, prms)
        '
        ' What I found was that the VB compiler did not recognize a GetType() method for IMenuHandler, but it worked fine
        ' in the C# version.  Through trial and error I found that there was a GetType() method recognized for
        ' niCurrent.GetService(GetType(IMenuHandler)), so I modified AddMenuItem to take an INodeInformation object,
        ' do the conversion to IMenuHandler here, and then do the rest of the line.
        '                                                                               - Mark Gilbert, 11/26/2006
        niCurrent.GetService(GetType(IMenuHandler)).GetType().GetMethod("AddChild").Invoke(niCurrent.GetService(GetType(IMenuHandler)), objParameters)
    End Sub

End Class
