' ***************************************************************************************************
' SSMS Scripter
' A plugin for SQL Server 2005 Management Studio that allows you to script stored procedures,
' user-defined functions, and views in the SQL 2000 Enterprise Manager
' style: if exists, drop; create. 
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' ***************************************************************************************************

Imports Microsoft.SqlServer.Management.Smo
Imports Microsoft.SqlServer.Management.Common
Imports System.Text
Imports System.IO
Imports System.Data.SqlClient

Public Class SSMSScriptor

    Private _CurrentOLAPConnection As Microsoft.SqlServer.Management.Common.SqlOlapConnectionInfoBase
    Private _ServerName As String
    Private _DatabaseName As String
    Private _CurrentDatabase As Database
    Private _CurrentServer As Server
    Private _DBObjectName As String
    Private _DBObjectSelectedNodeType As MenuNode.MenuNodeType


    ''' <summary>
    ''' This enumeration is used to control where the scripts are sent.
    ''' </summary>
    ''' <remarks></remarks>
    Private Enum DestinationForScriptsEnum
        NotSpecified = 0
        File = 1
        Clipboard = 2
        PreviewWindow = 3
    End Enum


    ''' <summary>
    ''' This event is raised when the Scripter has finished loading.
    ''' </summary>
    ''' <remarks></remarks>
    Public Event ScripterLoaded()


    ''' <summary>
    ''' The main constructor
    ''' </summary>
    ''' <param name="NewOLAPConnection">The SQL OLAP connection information for the database being scripted.</param>
    ''' <param name="NewDatabaseName">The name of the database being scripted.</param>
    ''' <param name="NewMenuNodeType">The type of entity that was used to invoke the SSMS Scripter.</param>
    ''' <param name="DefaultObjectName">The name of the entity used to invoke the SSMS Scripter.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NewOLAPConnection As Microsoft.SqlServer.Management.Common.SqlOlapConnectionInfoBase, _
                    ByVal NewDatabaseName As String, _
                    ByVal NewMenuNodeType As Integer, _
                    ByVal DefaultObjectName As String)
        InitializeComponent()

        ' Initialize the private variables
        Me._CurrentOLAPConnection = NewOLAPConnection
        Me.DatabaseName = NewDatabaseName
        Me._DBObjectSelectedNodeType = CType(NewMenuNodeType, MenuNode.MenuNodeType)
        Me._DBObjectName = DefaultObjectName

        ' Connect to the server specified
        If (IsNothing(Me.CurrentServer)) Then Throw New Exception("An error occurred trying to establish a connection to database '" & Me.DatabaseName & "'.  Please check the database name and make sure you have the appropriate permissiont to connect to it.")

        ' Connect to the database specified
        If (IsNothing(Me.CurrentDatabase)) Then Throw New Exception("Database '" & Me.DatabaseName & "' does not exist on server '" & Me.ServerName & "', or permission denied.")
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This provides access to the Server Name for the rest of the form.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property ServerName() As String
        Get
            If (IsNothing(Me._ServerName)) Then
                If (IsNothing(Me._CurrentOLAPConnection)) Then
                    Me._ServerName = ""
                Else
                    Me._ServerName = Me._CurrentOLAPConnection.ServerName
                End If
            End If

            Return Me._ServerName.Trim
        End Get
    End Property


    ''' <summary>
    ''' This provides access to the Database Name for the rest of the form.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Property DatabaseName() As String
        Get
            If (IsNothing(Me._DatabaseName)) Then Me._DatabaseName = ""
            Return Me._DatabaseName.Trim
        End Get
        Set(ByVal value As String)
            If (IsNothing(value)) Then value = ""
            Me._DatabaseName = value.Trim
        End Set
    End Property


    ''' <summary>
    ''' This provides access to the SMO Database object for the rest of the form.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>This will initialized the Database object if it isn't already.</remarks>
    Public ReadOnly Property CurrentDatabase() As Database
        Get
            If (Not IsNothing(Me._CurrentDatabase)) Then Return Me._CurrentDatabase

            Me._CurrentDatabase = Me.CurrentServer.Databases(Me.DatabaseName)
            Return Me._CurrentDatabase
        End Get
    End Property


    ''' <summary>
    ''' This provides access to the SMO Server object for the rest of the form.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' This will initialize the Server object if it isn't already.  It will
    ''' also force the IsSystemObject property to be returned.  This isn't part of
    ''' the default property set to be retrieved, so forcing it here will dramatically
    ''' improve performance later.
    ''' </remarks>
    Public ReadOnly Property CurrentServer() As Server
        Get
            If (Not IsNothing(Me._CurrentServer)) Then Return Me._CurrentServer
            If (IsNothing(Me._CurrentOLAPConnection)) Then Return Nothing

            Me._CurrentServer = New Server(New ServerConnection(New SqlConnection(Me._CurrentOLAPConnection.ConnectionString)))
            Me._CurrentServer.SetDefaultInitFields(GetType(StoredProcedure), "IsSystemObject")
            Me._CurrentServer.SetDefaultInitFields(GetType(View), "IsSystemObject")
            Me._CurrentServer.SetDefaultInitFields(GetType(UserDefinedFunction), "IsSystemObject")
            Me._CurrentServer.ConnectionContext.SqlExecutionModes = Microsoft.SqlServer.Management.Common.SqlExecutionModes.CaptureSql
            Return Me._CurrentServer
        End Get
    End Property


    ''' <summary>
    ''' This returns True if all of the text fields have something in them.  If one
    ''' or more of these is blank, this property returns False.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsValidForm() As Boolean
        Get
            Return Me.txtDatabaseName.Text.Trim.Length > 0 And _
                    Me.txtServerName.Text.Trim.Length > 0 And _
                    ( _
                        (Me.radScriptToFile.Checked And Me.txtScriptingFilePath.Text.Trim.Length > 0) Or _
                        Me.radScriptToClipboard.Checked Or _
                        Me.radPreview.Checked _
                    )
        End Get
    End Property


    ''' <summary>
    ''' This private property allows easy an easy way to determine where the scripts should go once generated.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property DestinationForScripts() As DestinationForScriptsEnum
        Get
            If (Me.radScriptToFile.Checked) Then Return DestinationForScriptsEnum.File
            If (Me.radScriptToClipboard.Checked) Then Return DestinationForScriptsEnum.Clipboard
            If (Me.radPreview.Checked) Then Return DestinationForScriptsEnum.PreviewWindow

            Return DestinationForScriptsEnum.NotSpecified
        End Get
    End Property


    ''' <summary>
    ''' This private property exposes the default object name passed into the class from SSMS.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property DefaultObjectName() As String
        Get
            If (IsNothing(Me._DBObjectName)) Then Me._DBObjectName = ""
            Return Me._DBObjectName.Trim
        End Get
    End Property
#End Region


#Region " Methods "


#Region " Progress Bar Methods "
    ''' <summary>
    ''' This initializes the progress bar's value, and turns it visible.  It also initializes
    ''' the progress bar label.
    ''' </summary>
    ''' <param name="strLabel">
    ''' The text to appear with the progress bar.  If "nothing" is passed in for this parameter,
    ''' the label and progress bar are cleared out and hidden.
    ''' </param>
    ''' <remarks></remarks>
    Private Sub InitializeProgressBar(ByVal strLabel As String)
        Me.InitializeProgressBar(strLabel, False)
    End Sub


    ''' <summary>
    ''' This variant will allow the calling code to force the label to appear without the progress bar being visible.
    ''' </summary>
    ''' <param name="strLabel"></param>
    ''' <param name="blnHideProgressBar"></param>
    ''' <remarks></remarks>
    Private Sub InitializeProgressBar(ByVal strLabel As String, ByVal blnHideProgressBar As Boolean)
        ' Reset the progress bar
        Me.pbarMain.Value = 0

        ' Set visibiliy for the two controls
        Me.pbarMain.Visible = Not IsNothing(strLabel) And Not blnHideProgressBar
        Me.lblProgress.Visible = Not IsNothing(strLabel)

        ' Set the label text if it is visible
        If (Not IsNothing(strLabel)) Then Me.lblProgress.Text = IIf(strLabel.Trim.Length = 0, "Progress:", strLabel)

        ' Do some events to let the interface repaint
        Application.DoEvents()
    End Sub

    ''' <summary>
    ''' This initializes the Step property of the progress bar, the width that the bar should move
    ''' each time the PerformStep method is called.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetNumberOfProgressBarSteps()
        Dim intNumberOfSteps As Integer
        Dim tnCurrent As TreeNode

        intNumberOfSteps = 0
        For Each tnCurrent In Me.tvwDBObjects.Nodes
            intNumberOfSteps += Me.CountCheckedObjects(tnCurrent.Nodes)
        Next

        ' If nothing was selected, set the progress bar up for a single step
        If (intNumberOfSteps = 0) Then intNumberOfSteps = 1
        Me.pbarMain.Step = Me.pbarMain.Width / intNumberOfSteps
    End Sub


    ''' <summary>
    ''' This counts the number of checked nodes in the current branch, and any sub-branches.
    ''' </summary>
    ''' <param name="tncCurrent">The tree branch to dig into, looking for checked nodes</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CountCheckedObjects(ByVal tncCurrent As TreeNodeCollection) As Integer
        Dim tnCurrent As TreeNode
        Dim intCheckedNodes As Integer

        intCheckedNodes = 0
        For Each tnCurrent In tncCurrent
            intCheckedNodes += IIf(tnCurrent.Checked, 1, 0)
            If (tnCurrent.Nodes.Count > 0) Then intCheckedNodes += Me.CountCheckedObjects(tnCurrent.Nodes)
        Next

        Return intCheckedNodes
    End Function


    ''' <summary>
    ''' This moves the progress bar the specified number of steps.
    ''' </summary>
    ''' <param name="intNumberOfStepsToMove">The number of steps that the progress bar should be moved.</param>
    ''' <remarks></remarks>
    Private Sub MoveProgressBarForward(ByVal intNumberOfStepsToMove As Integer)
        Dim i As Integer

        If (intNumberOfStepsToMove <= 0) Then Return

        For i = 1 To intNumberOfStepsToMove
            Me.pbarMain.PerformStep()
            Application.DoEvents()
        Next
    End Sub
#End Region


    ''' <summary>
    ''' This generates any necessary scripts for the current tree node, and any of its children.
    ''' </summary>
    ''' <param name="tncCurrentBranch">The branch of the Tree to start scripting from.</param>
    ''' <param name="CurrentScripter">The SMO Scripter object to use to script out the objects, pre-configured.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GenerateScripts(ByVal tncCurrentBranch As TreeNodeCollection, ByVal CurrentScripter As Scripter) As String
        Dim stnCurrent As ScripterTreeNode
        Dim sbScripts As StringBuilder

        If (IsNothing(tncCurrentBranch)) Then Return ""

        sbScripts = New StringBuilder
        For Each stnCurrent In tncCurrentBranch
            ' If the current node is checked, script it
            If (stnCurrent.Checked) Then sbScripts.Append(stnCurrent.ToScript(CurrentScripter))

            ' Then see if the current node has children, and script those recursively
            If (stnCurrent.Nodes.Count > 0) Then sbScripts.Append(Me.GenerateScripts(stnCurrent.Nodes, CurrentScripter))

            Me.MoveProgressBarForward(1)
        Next

        Return sbScripts.ToString
    End Function


    ''' <summary>
    ''' This sets up the form fields based on what was passed into the application from SSMS.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub InitializeFormControls()

        Me.txtServerName.Enabled = False
        Me.txtServerName.Text = Me.ServerName

        Me.txtDatabaseName.Enabled = False
        Me.txtDatabaseName.Text = Me.DatabaseName

        ' Populate the treeview with the necessary database objects
        Me.tvwDBObjects.CheckBoxes = True
        Me.AttachTopLevelBranchesToTreeView()
    End Sub


    ''' <summary>
    ''' This method adds the top-level branches to the treeview, and a single dummy child
    ''' node under each (to force the expansion icon to appear).
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AttachTopLevelBranchesToTreeView()
        Me.tvwDBObjects.Nodes.Add(New ScripterTreeNode("Views", ScripterTreeNode.ScripterTreeNodeType.ViewsParent, Nothing, Me.CurrentDatabase.Views))
        Me.tvwDBObjects.Nodes.Add(New ScripterTreeNode("Stored Procedures", ScripterTreeNode.ScripterTreeNodeType.StoredProceduresParent, Nothing, Me.CurrentDatabase.StoredProcedures))
        Me.tvwDBObjects.Nodes.Add(New ScripterTreeNode("User Defined Functions", ScripterTreeNode.ScripterTreeNodeType.UserDefinedFunctionsParent, Nothing, Me.CurrentDatabase.UserDefinedFunctions))
    End Sub


    ''' <summary>
    ''' This method looks at the IsValidForm property, and based in the results will enable or disable
    ''' the Generate button.  The form must be valid before the button will be enabled.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ValidateForm()
        Me.btnGenerate.Enabled = Me.IsValidForm()
    End Sub


    ''' <summary>
    ''' This displays the File Save dialog box.  When that dialog closes, the form is revalidated, and
    ''' focus is moved to the Generate button if the form is valid.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DisplayFileSaveDialog()
        Dim ofsBrowse As New System.Windows.Forms.SaveFileDialog

        ' Filter the dialog, and disable multiselect
        ofsBrowse.Filter = "SQL Server Scripts (*.sql)|*.sql"

        ' Get the selected file if they selected one
        If ofsBrowse.ShowDialog() = Windows.Forms.DialogResult.OK Then Me.txtScriptingFilePath.Text = ofsBrowse.FileName

        ' Validate the form
        Me.ValidateForm()

        ' Move the focus to the Generate button, if enabled
        If (Me.btnGenerate.Enabled) Then Me.btnGenerate.Focus()
    End Sub


    ''' <summary>
    ''' This enables/disables the controls on the form that were enabled originally so that the
    ''' user doesn't get into trouble while the scripter is running.
    ''' </summary>
    ''' <param name="blnShouldBeEnabled"></param>
    ''' <remarks></remarks>
    Private Sub EnableControlsForGeneration(ByVal blnShouldBeEnabled As Boolean)
        Me.txtScriptingFilePath.Enabled = blnShouldBeEnabled And Me.radScriptToFile.Checked
        Me.btnScriptingBrowse.Enabled = blnShouldBeEnabled And Me.radScriptToFile.Checked
        Me.btnGenerate.Enabled = blnShouldBeEnabled And Me.IsValidForm
        Me.btnCancel.Enabled = blnShouldBeEnabled
        Me.radScriptToFile.Enabled = blnShouldBeEnabled
        Me.radScriptToClipboard.Enabled = blnShouldBeEnabled
        Me.radPreview.Enabled = blnShouldBeEnabled
    End Sub


    ''' <summary>
    ''' This writes out the scripts to the destination (file, clipboard, etc.).
    ''' </summary>
    ''' <param name="strScripts">The scripts to write out.</param>
    ''' <remarks></remarks>
    Private Sub SendScriptsToDestination(ByVal strScripts As String)
        Dim swScripts As StreamWriter
        Dim frmScriptPreview As ScriptPreview

        ' Check to make sure strScripts has something in it; if not, display an error message
        If (IsNothing(strScripts)) Then strScripts = ""
        If (strScripts.Trim.Length = 0) Then Throw New Exception("Either no objects were selected to be scripted, or no scripts were possible for the objects selected.")

        ' MG 12/11/2006: I was having problems with a select few stored procedures in one of our databases.  The carriage
        ' returns were not vbCrLf, and therefore the Preview Window wouldn't show the lines formatted correctly.  Also,
        ' opening the clipboard contents, or the file contents in Notepad would not show the lines formatted correctly.
        ' Textpad, however, was able to correct for this.
        '
        ' After writing a quick little application to translate the characters that I saw in Notepad to their ASCII
        ' equivalents, I determined that the problems were being caused by line feeds (ASCII 10) not being preceeded
        ' by carriage returns (ASCII 13).  So, this code corrects that by replacing any line feeds with a carriage
        ' return line feed combination, and then replacing any double carriage returns with a single carriage return.
        ' Due to the nature and order of these replacements, any valid CRLF combinations are effectively ignored.
        strScripts = strScripts.Replace(vbLf, vbCrLf)
        strScripts = strScripts.Replace(vbCr & vbCr, vbCr)

        Select Case Me.DestinationForScripts
            Case DestinationForScriptsEnum.File
                swScripts = File.AppendText(Me.txtScriptingFilePath.Text)
                swScripts.WriteLine(strScripts)
                swScripts.Close()

            Case DestinationForScriptsEnum.Clipboard
                Clipboard.SetText(strScripts)

            Case DestinationForScriptsEnum.PreviewWindow
                frmScriptPreview = New ScriptPreview(strScripts)
                frmScriptPreview.ShowDialog()

            Case Else
                Throw New Exception("Invalid destination for scripts specified.")
        End Select
    End Sub


    ''' <summary>
    ''' This will load the parent node's children if they haven't already.  Most of the work of
    ''' this is done by the AddChildren method, but LoadChildren wraps this call in Me.Cursor
    ''' assignments to turn the mousepointer to an hourglass and back.
    ''' </summary>
    ''' <param name="stnParent">The parent node to load children for</param>
    ''' <remarks></remarks>
    Private Sub LoadChildren(ByVal stnParent As ScripterTreeNode)
        If (Not stnParent.AreChildrenLoaded) Then
            Me.Cursor = Cursors.WaitCursor
            stnParent.AddChildren()
            Me.Cursor = Cursors.Default
        End If
    End Sub


    ''' <summary>
    ''' This will return the ScripterTreeNodeType equivalent to the MenuNodeType value passed in.  
    ''' </summary>
    ''' <param name="intCurrentMenuNodeType">
    ''' The integer version of the MenuNode.MenuNodeType value to convert to a ScripterTreeNodeType.
    ''' </param>
    ''' <returns></returns>
    ''' <remarks>
    ''' Even though intCurrentMenuNodeType is of type MenuNode.MenuNodeTypeEnum, when I tried to
    ''' use that type in the constructor argument list, I was getting this error: "'intCurrentMenuNodeType'
    ''' can't expose type 'MenuNode.MenuNodeTypeEnum' outside of class 'SSMSScriptor'".  It's
    ''' because the MenuNode class is declared as Friend.
    ''' </remarks>
    Private Function ConvertMenuNodeTypeToScripterTreeNodeParentType(ByVal intCurrentMenuNodeType As Integer) As ScripterTreeNode.ScripterTreeNodeType
        Select Case CType(intCurrentMenuNodeType, MenuNode.MenuNodeType)
            Case MenuNode.MenuNodeType.Views
                Return ScripterTreeNode.ScripterTreeNodeType.ViewsParent

            Case MenuNode.MenuNodeType.StoredProcedures
                Return ScripterTreeNode.ScripterTreeNodeType.StoredProceduresParent

            Case MenuNode.MenuNodeType.UserDefinedFunctions
                Return ScripterTreeNode.ScripterTreeNodeType.UserDefinedFunctionsParent

            Case MenuNode.MenuNodeType.Database
                Return ScripterTreeNode.ScripterTreeNodeType.All

            Case Else
                Return ScripterTreeNode.ScripterTreeNodeType.None
        End Select
    End Function
#End Region


#Region " Event Handlers "
    Private Sub btnScriptingBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnScriptingBrowse.Click
        Me.DisplayFileSaveDialog()
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.Close()
    End Sub

    Private Sub btnGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGenerate.Click
        Dim strScripts As String
        Dim MyScripter As Scripter

        Try
            ' If the user has selected a file to send the scripts to, see if it already exists; if so, prompt them to overwrite
            If Me.DestinationForScripts = DestinationForScriptsEnum.File Then
                If (File.Exists(Me.txtScriptingFilePath.Text)) Then
                    If (MsgBox("The script file that you specified already exists.  Would you like to overwrite it?", MsgBoxStyle.YesNo, "SSMS Scripter") = MsgBoxResult.Yes) Then
                        File.Delete(Me.txtScriptingFilePath.Text)
                    Else
                        Exit Try
                    End If
                End If
            End If

            ' Generation initialization
            Me.Cursor = Cursors.WaitCursor
            Me.EnableControlsForGeneration(False)
            Me.SetNumberOfProgressBarSteps()

            ' Initialize the string variable that will hold all of the scripts
            strScripts = ""

            ' First, collect the DROP statements
            Me.InitializeProgressBar("Generating DROP Statements (Step 1 of 2):")
            MyScripter = New Scripter(Me.CurrentServer)
            MyScripter.Options.ScriptDrops = True
            MyScripter.Options.IncludeIfNotExists = True
            strScripts &= Me.GenerateScripts(Me.tvwDBObjects.Nodes, MyScripter)
            MyScripter = Nothing

            ' Now, collect the CREATE statements
            Me.InitializeProgressBar("Generating CREATE Statements (Step 2 of 2):")
            MyScripter = New Scripter(Me.CurrentServer)
            MyScripter.Options.PrimaryObject = True
            strScripts &= Me.GenerateScripts(Me.tvwDBObjects.Nodes, MyScripter)
            MyScripter = Nothing


            Select Case Me.DestinationForScripts
                Case DestinationForScriptsEnum.Clipboard
                    Me.SendScriptsToDestination(strScripts)
                    Me.InitializeProgressBar("Scripts copied to clipboard.", True)
                Case DestinationForScriptsEnum.File
                    Me.SendScriptsToDestination(strScripts)
                    Me.InitializeProgressBar("Scripts saved to file.", True)
                Case DestinationForScriptsEnum.PreviewWindow
                    Me.SendScriptsToDestination(strScripts)
                    Me.InitializeProgressBar(Nothing)
                Case Else
                    Me.InitializeProgressBar(Nothing)
            End Select

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.OkOnly, "SSMS Scripter Error")

        Finally
            Me.EnableControlsForGeneration(True)
            Me.Cursor = Cursors.Default

        End Try
    End Sub


    Private Sub SSMSScriptor_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim stnParentToBeDefaulted As ScripterTreeNode
        Dim stntDefaultObject As ScripterTreeNode.ScripterTreeNodeType

        Me.InitializeFormControls()
        Me.ValidateForm()

        ' Now, default the object(s) selected by the user in SSMS.  If a valid ScripterTreeNodeType can't
        ' be determined, then just skip this step.
        ' TODO: This is the only place that Me.ConvertMenuNodeTypeToScripterTreeNodeParentType is used...
        '       There should be a better way to do this.
        stntDefaultObject = Me.ConvertMenuNodeTypeToScripterTreeNodeParentType(Me._DBObjectSelectedNodeType)
        If (stntDefaultObject = ScripterTreeNode.ScripterTreeNodeType.None) Then Return
        If (stntDefaultObject = ScripterTreeNode.ScripterTreeNodeType.All) Then
            ' Expand and check all objects in the tree view
            For Each stnParentToBeDefaulted In Me.tvwDBObjects.Nodes
                stnParentToBeDefaulted.Expand()
                stnParentToBeDefaulted.Checked = True
            Next
        Else
            ' Determine the parent node to be expanded
            stnParentToBeDefaulted = Me.tvwDBObjects.Nodes(stntDefaultObject)

            ' Expand the parent node; the expansion also builds out the node's children
            stnParentToBeDefaulted.Expand()

            ' If "All Objects" have been selected to be defaulted, then just check the parent, and the tvwDBObjects_AfterCheck
            ' event handler will check all of the child nodes.  If a specific object was selected, just check that one.
            If (Me.DefaultObjectName = MenuNode.ALL_OBJECTS) Then
                stnParentToBeDefaulted.Checked = True
            Else
                stnParentToBeDefaulted.CheckChild(Me.DefaultObjectName)
            End If
        End If

        RaiseEvent ScripterLoaded()
    End Sub


    Private Sub DatabaseFields_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtDatabaseName.KeyUp, _
                                                                                                                    txtServerName.KeyUp, _
                                                                                                                    txtScriptingFilePath.KeyUp
        Me.ValidateForm()
    End Sub


    Private Sub RadioButtons_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radScriptToFile.CheckedChanged, _
                                                                                                                radScriptToClipboard.CheckedChanged, _
                                                                                                                radPreview.CheckedChanged
        Me.txtScriptingFilePath.Enabled = Me.radScriptToFile.Checked
        Me.btnScriptingBrowse.Enabled = Me.radScriptToFile.Checked

        Me.ValidateForm()
    End Sub


    ''' <summary>
    ''' If the user checked a parent node, then all child nodes should be checked.  Ditto with
    ''' clearing a parent node.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tvwDBObjects_AfterCheck(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles tvwDBObjects.AfterCheck
        Dim stnParent, stnChild As ScripterTreeNode

        stnParent = CType(e.Node, ScripterTreeNode)
        Me.LoadChildren(stnParent)

        For Each stnChild In stnParent.Nodes
            stnChild.Checked = stnParent.Checked
        Next
    End Sub


    Private Sub tvwDBObjects_AfterExpand(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles tvwDBObjects.AfterExpand
        Dim stnParent As ScripterTreeNode

        stnParent = CType(e.Node, ScripterTreeNode)
        Me.LoadChildren(stnParent)
    End Sub
#End Region


End Class
