' ***************************************************************************************************
' SSMS Scripter
' A plugin for SQL Server 2005 Management Studio that allows you to script stored procedures,
' user-defined functions, and views in the SQL 2000 Enterprise Manager
' style: if exists, drop; create. 
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' ***************************************************************************************************

Imports Microsoft.SqlServer.Management.Smo
Imports System.Text
Imports System.Collections.Specialized

Public Class ScripterTreeNode
    Inherits System.Windows.Forms.TreeNode

    ' *******************************************************************************************
    ' Enum:         ScripterTreeNodeType
    ' Author:       Mark Gilbert
    ' Date:         {unknown}
    ' Description:  This enumerates the types of tree nodes the Scripter's Tree View is designed
    '               to work with.
    ' *******************************************************************************************
    Public Enum ScripterTreeNodeType
        ViewsParent = 0
        StoredProceduresParent = 1
        UserDefinedFunctionsParent = 2
        View = 3
        StoredProcedure = 4
        UserDefinedFunction = 5
        All = 6
        None = 7
    End Enum


    Private Const DUMMY_NODE_TEXT As String = "{dummy}"


    Private m_stntCurrent As ScripterTreeNodeType
    Private m_urnCurrent As Microsoft.SqlServer.Management.Smo.Urn
    Private m_blnAreChildrenLoaded As Boolean
    Private m_colObjects As Microsoft.SqlServer.Management.Smo.SchemaCollectionBase


    ' *******************************************************************************************
    ' Property:     New
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   strLabel - the label for the new node
    '               otCurrent - the object type for the new node
    '               urnCurrent - the Uniform Resource Name for the new node
    '               otDefault - the type of object that should be initially checked.  This
    '                   corresponds to the type of object that the user selected in SSMS.
    '               strDefaultObjectName - the name of the default object, or Common.ALL_OBJECTS
    '                   if all objects of a given type have been selected in SSMS.
    '               colObjects - the collection of SMO objects to populate the list with
    '                   initially.  This is really only used if the user has selected a specific
    '                   object, or a specific folder.
    ' Description:  This property instantiates the new node, and sets its label and object type
    ' *******************************************************************************************
    Public Sub New(ByVal strLabel As String, _
                    ByVal stntCurrent As ScripterTreeNodeType, _
                    ByVal urnCurrent As Microsoft.SqlServer.Management.Smo.Urn, _
                    ByVal colObjects As Microsoft.SqlServer.Management.Smo.SchemaCollectionBase)
        MyBase.New()

        Dim stntDummyChild As ScripterTreeNodeType

        ' Set up the private class properties
        Me.Text = strLabel
        Me.m_stntCurrent = stntCurrent
        Me.m_urnCurrent = urnCurrent
        Me.m_blnAreChildrenLoaded = False
        Me.m_colObjects = colObjects

        ' If this is a top level (parent) node, add the dummy node
        If (Me.IsParentNode) Then
            stntDummyChild = Me.ConvertTypeParentToChild(stntCurrent)
            Me.Nodes.Add(New ScripterTreeNode(DUMMY_NODE_TEXT, stntDummyChild, Nothing, Nothing))
        End If

    End Sub


#Region " Properties "
    ' *******************************************************************************************
    ' Property:     IsObjectToBeScripted
    ' Author:       Mark Gilbert
    ' Date:         12/23/2006
    ' Parameters:   {none}
    ' Description:  This will return True if the current object type is View, StoredProcedure,
    '               or UserDefinedFunction.  It will return False otherwise.
    ' *******************************************************************************************
    Public ReadOnly Property IsObjectToBeScripted() As Boolean
        Get
            Return (Me.m_stntCurrent = ScripterTreeNodeType.View Or _
                    Me.m_stntCurrent = ScripterTreeNodeType.StoredProcedure Or _
                    Me.m_stntCurrent = ScripterTreeNodeType.UserDefinedFunction)
        End Get
    End Property


    ' *******************************************************************************************
    ' Property:     Urn
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This property exposes the Urn (Uniform Resource Name) value for the
    '               database object.
    ' *******************************************************************************************
    Public Property Urn() As Microsoft.SqlServer.Management.Smo.Urn
        Get
            Return Me.m_urnCurrent
        End Get
        Set(ByVal value As Microsoft.SqlServer.Management.Smo.Urn)
            Me.m_urnCurrent = value
        End Set
    End Property


    ' *******************************************************************************************
    ' Property:     AreChildrenLoaded
    ' Author:       Mark Gilbert
    ' Date:         12/23/2006
    ' Parameters:   {none}
    ' Description:  This property exposes the m_blnAreChildrenLoaded property.
    ' *******************************************************************************************
    Public ReadOnly Property AreChildrenLoaded() As Boolean
        Get
            Return Me.m_blnAreChildrenLoaded
        End Get
    End Property


    ' *******************************************************************************************
    ' Property:     IsParentNode
    ' Author:       Mark Gilbert
    ' Date:         12/23/2006
    ' Parameters:   {none}
    ' Description:  This property returns True if the current node is one of the "parent" nodes;
    '               it returns False otherwise.
    ' *******************************************************************************************
    Public ReadOnly Property IsParentNode() As Boolean
        Get
            Return (Me.m_stntCurrent = ScripterTreeNodeType.ViewsParent Or _
                    Me.m_stntCurrent = ScripterTreeNodeType.StoredProceduresParent Or _
                    Me.m_stntCurrent = ScripterTreeNodeType.UserDefinedFunctionsParent)
        End Get
    End Property


    ' *******************************************************************************************
    ' Property:     NodeType
    ' Author:       Mark Gilbert
    ' Date:         12/25/2006
    ' Parameters:   {none}
    ' Description:  This property exposes the m_otCurrent variable, which describes the type
    '               of the current node.
    ' *******************************************************************************************
    Public ReadOnly Property NodeType() As ScripterTreeNodeType
        Get
            Return Me.m_stntCurrent
        End Get
    End Property
#End Region


#Region " Methods "

    ' *******************************************************************************************
    ' Method:       ConvertTypeParentToChild
    ' Author:       Mark Gilbert
    ' Date:         12/23/2006
    ' Parameters:   otSource - the ObjectTypeEnum source value
    ' Description:  The ObjectTypeEnum enumeration contains a list of parent and child object
    '               types.  This function returns the equivalent child type for the parent type
    '               passed in.
    ' *******************************************************************************************
    Private Function ConvertTypeParentToChild(ByVal stntSource As ScripterTreeNodeType) As ScripterTreeNodeType
        Select Case stntSource
            Case ScripterTreeNodeType.ViewsParent
                Return ScripterTreeNodeType.View

            Case ScripterTreeNodeType.StoredProceduresParent
                Return ScripterTreeNodeType.StoredProcedure

            Case ScripterTreeNodeType.UserDefinedFunctionsParent
                Return ScripterTreeNodeType.UserDefinedFunction

        End Select
    End Function


    ' *******************************************************************************************
    ' Method:       ToScript
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   scrCurrent - the current Scripter object to use to create the scripts
    ' Description:  This generates the scripts for the database object represented by the
    '               current node, or a zero-length string if the object type is "Other".  This
    '               code was originally part of SSMSScripter, but was moved here as the result
    '               of a refactoring.
    ' Revisions:
    '   12/23/2006  Mark Gilbert updated the code to use the new IsObjectToBeScripted property.
    ' *******************************************************************************************
    Public Function ToScript(ByVal scrCurrent As Scripter) As String
        Dim smoObjects(1) As Urn
        Dim scScripts As StringCollection
        Dim strCurrentScript As String
        Dim sbScripts As StringBuilder

        ' If this isn't an object that should be scripted, then just exit
        If (Not Me.IsObjectToBeScripted) Then Return ""

        smoObjects = New Urn(0) {}
        smoObjects(0) = Me.Urn

        scScripts = scrCurrent.Script(smoObjects)
        sbScripts = New StringBuilder
        For Each strCurrentScript In scScripts
            sbScripts.Append(strCurrentScript)
            sbScripts.Append(vbCrLf & "GO" & vbCrLf & vbCrLf)
        Next

        Return sbScripts.ToString
    End Function



    ' *******************************************************************************************
    ' Method:       AttachBranchForViews
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This method adds the Views branch to the treeview.
    ' Revisions:
    '   12/23/2006  Mark Gilbert moved this method from SSMSScripter to here
    ' *******************************************************************************************
    Private Sub AttachBranchForViews()
        Dim vwCurrent As View

        ' Add the children nodes, and check them if they were selected by the user in SSMS
        For Each vwCurrent In CType(Me.m_colObjects, ViewCollection)
            If (Not vwCurrent.IsSystemObject) Then
                Me.Nodes.Add(New ScripterTreeNode(vwCurrent.Name, ScripterTreeNodeType.View, vwCurrent.Urn, Nothing))
            End If
        Next
    End Sub


    ' *******************************************************************************************
    ' Method:       AttachBranchForStoredProcedures
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This method adds the Stored Procedures branch to the treeview.
    ' Revisions:
    '   12/23/2006  Mark Gilbert moved this method from SSMSScripter to here
    ' *******************************************************************************************
    Private Sub AttachBranchForStoredProcedures()
        Dim spCurrent As StoredProcedure

        ' Add the children nodes, and check them if they were selected by the user in SSMS
        For Each spCurrent In CType(Me.m_colObjects, StoredProcedureCollection)
            If (Not spCurrent.IsSystemObject) Then
                Me.Nodes.Add(New ScripterTreeNode(spCurrent.Name, ScripterTreeNodeType.StoredProcedure, spCurrent.Urn, Nothing))
            End If
        Next
    End Sub


    ' *******************************************************************************************
    ' Method:       AttachBranchForUserDefinedFunctions
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This method adds the User-Defined Functions branch to the treeview.
    ' Revisions:
    '   12/23/2006  Mark Gilbert moved this method from SSMSScripter to here
    ' *******************************************************************************************
    Private Sub AttachBranchForUserDefinedFunctions()
        Dim udfCurrent As UserDefinedFunction

        ' Add the children nodes, and check them if they were selected by the user in SSMS
        For Each udfCurrent In CType(Me.m_colObjects, UserDefinedFunctionCollection)
            If (Not udfCurrent.IsSystemObject) Then
                Me.Nodes.Add(New ScripterTreeNode(udfCurrent.Name, ScripterTreeNodeType.UserDefinedFunction, udfCurrent.Urn, Nothing))
            End If
        Next
    End Sub


    ' *******************************************************************************************
    ' Method:       AddChildren
    ' Author:       Mark Gilbert
    ' Date:         12/23/2006
    ' Parameters:   {none}
    ' Description:  This adds the children nodes for the current one, if any.  It also checks
    '               the objects specified by the input parameters.
    ' *******************************************************************************************
    Public Sub AddChildren()
        If (Not Me.IsParentNode) Then Return

        Me.Nodes.Clear()
        Select Case Me.m_stntCurrent
            Case ScripterTreeNodeType.ViewsParent
                Me.AttachBranchForViews()

            Case ScripterTreeNodeType.StoredProceduresParent
                Me.AttachBranchForStoredProcedures()

            Case ScripterTreeNodeType.UserDefinedFunctionsParent
                Me.AttachBranchForUserDefinedFunctions()

        End Select
        Me.m_blnAreChildrenLoaded = True
    End Sub


    ' *******************************************************************************************
    ' Method:       CheckChild
    ' Author:       Mark Gilbert
    ' Date:         12/25/2006
    ' Parameters:   strDefaultName - the text of the node to Check
    ' Description:  This checks the child node with the same Text as the parameter passed in.
    ' *******************************************************************************************
    Public Sub CheckChild(ByVal strDefaultName As String)
        Dim stnCurrentChild As ScripterTreeNode

        For Each stnCurrentChild In Me.Nodes
            If (stnCurrentChild.Text = strDefaultName) Then
                stnCurrentChild.Checked = True
                Return
            End If
        Next

    End Sub

#End Region

End Class
